#! /usr/bin/env python
#
# Version 1.0  2016-09-20
#
# XCLASSViewer.py        -- Peter Schilke (<schilke@ph1.uni-koeln.de>)
#

from PyQt4.uic import loadUiType
import numpy as np
 
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt4agg import (
    FigureCanvasQTAgg as FigureCanvas,
    NavigationToolbar2QT as NavigationToolbar)

import os  # For listing directory methods
import sys
import numpy as np
#import seaborn as sbn
from scipy.constants import codata
C = codata.physical_constants
c = C['speed of light in vacuum'][0]
k = C['Boltzmann constant'][0]
h = C['Planck constant'][0]
# extend sys.path variable

#customize the following path

XCLASSHome='/home/schilke/software/XCLASS-Interface'
XCLASSHelpers='/home/schilke/software/XCLASS-Helpers'


NewModulesPath = XCLASSHome+'/build_tasks/'


already_included_flag = "false"
for entries in sys.path:
    if (entries == NewModulesPath):
        already_included_flag = "true"
        break
    if (already_included_flag == "false"):
        sys.path.append(NewModulesPath)

import task_LoadASCIIFile as tL 
import task_myXCLASS  as tX

# customize the following path
Ui_MainWindow, QMainWindow = loadUiType(XCLASSHelpers+'/XCLASSViewer.ui')

class Main(QMainWindow, Ui_MainWindow):
    def __init__(self, ):
        super(Main, self).__init__()
        self.setupUi(self)
        self.setWindowTitle("XCLASS Viewer");


        self.dfreq = 0.0
        self.dataBrowse.clicked.connect(self.data_browse)  # When the button is pressed
                                                            # Execute data_browse function
        self.molfitBrowse.clicked.connect(self.molfit_browse)  # When the button is pressed
                                                            # Execute molfit_browse function
        self.molfitClear.clicked.connect(self.molfit_clear)  # When the button is pressed
                                                            # Execute molfit_browse function
        self.execFit_1.clicked.connect(self.exec_fit_1)         # When the button is pressed
                                                            # Execute exec_fit function
        self.execFit_2.clicked.connect(self.exec_fit_2)         # When the button is pressed
                                                            # Execute exec_fit function
        self.execFit_3.clicked.connect(self.exec_fit_3)         # When the button is pressed
                                                            # Execute exec_fit function
        self.execFit_4.clicked.connect(self.exec_fit_4)         # When the button is pressed
                                                            # Execute exec_fit function
#        self.CenterSlider.valueChanged.connect(self.set_center)
#        self.WidthSlider.valueChanged.connect(self.set_width)
        self.isoSelect.clicked.connect(self.get_iso)
        self.SizeValue.editingFinished.connect(self.get_size)
        self.ContLevel.editingFinished.connect(self.get_cont)
        self.inputvLSR.editingFinished.connect(self.get_vlsr)
        self.FreqMin.editingFinished.connect(self.get_fmin)
        self.FreqMax.editingFinished.connect(self.get_fmax)
        self.FreqStep.editingFinished.connect(self.get_fstep)
# set defaults
        self.contLevel = 0
        self.dfmin = 100000
        self.dfmax = 110000
        self.dfstep = 1
        self.sizeValue = 0.5
        self.isoFileName = "none"
        self.vLSR = 0
        self.ContLevel.setText(str(self.contLevel))
        self.inputvLSR.setText(str(self.vLSR))
        self.FreqMin.setText(str(self.dfmin))
        self.FreqMax.setText(str(self.dfmax))
        self.FreqStep.setText(str(self.dfstep))
        self.SizeValue.setText(str(self.sizeValue))
        self.isoValue.setText(self.isoFileName)
        self.interButton.isDown()

    def addmpl(self, fig):
        self.canvas = FigureCanvas(fig)
        self.mplvl.addWidget(self.canvas)
        self.canvas.draw()
        self.toolbar = NavigationToolbar(self.canvas, 
                self.mplwindow, coordinates=True)
        self.mplvl.addWidget(self.toolbar)



    def data_browse(self):
        self.DataList.clear() # In case there are any existing elements in the list
        self.DataFile = QtGui.QFileDialog.getOpenFileNames(self,
                                                           "Data File")
        # execute getExistingDirectory dialog and set the directory variable to be equal
        # to the user selected directory
        if self.DataFile: # if user didn't pick a file don't continue
            DataFileName = str(self.DataFile[0])
            self.DataList.addItem(DataFileName)
            NumHeaderLines = 2
            self.expdata = tL.LoadASCIIFile(DataFileName, NumHeaderLines, 0, 0)
        else:
           print "No Data file selected!"

        self.dfreq = self.expdata[:,0]
        self.data = self.expdata[:,1]
        self.dfmin = self.dfreq.min()
        self.dfmax = self.dfreq.max()
        self.dfwid = self.dfmax-self.dfmin
        self.dfmid = (self.dfmin+self.dfmax)/2
        self.dfstep = abs(self.dfreq[1]-self.dfreq[0])
        self.dmean = self.data.mean()
        minwid = 10
        self.first_plot()
#        self.CenterSlider.setMinimum(self.dfmin)
#        self.CenterSlider.setMaximum(self.dfmax)
#        self.CenterSlider.setValue(self.dfmid)
#        self.WidthSlider.setMaximum(self.dfwid)
#        self.WidthSlider.setMinimum(minwid)
#        self.WidthSlider.setValue(self.dfwid)
#        self.CenterSlider.valueChanged.connect(self.re_plot)
#        self.WidthSlider.valueChanged.connect(self.re_plot)
        self.contLevel = self.dmean
        self.ContLevel.setText(str(self.dmean))
        self.FreqMin.setText(str(int(self.dfmin)))
        self.FreqMax.setText(str(int(self.dfmax)))
        self.FreqStep.setText(str(self.dfstep))

    def molfit_clear(self):
        self.MolfitList.clear() # In case there are any existing elements in the list

    def molfit_browse(self):
        self.MolfitFile = QtGui.QFileDialog.getOpenFileNames(self,
                                                           "Molfit File")
        # execute getExistingDirectory dialog and set the directory variable to be equal
        # to the user selected directory

        if self.MolfitFile: # if user didn't pick a file don't continue
            self.MolFileName = str(self.MolfitFile[0])
            self.MolfitList.addItem(self.MolFileName)
        else:
            self.Status.addItem("No Molfit file selected!")
            self.Status.scrollToBottom()
            print "No Molfit file selected!"
  
    def get_iso(self):
        self.isoFile = QtGui.QFileDialog.getOpenFileNames(self, "Iso File")
        # execute getExistingDirectory dialog and set the directory variable to be equal
        # to the user selected directory

        if self.isoFile: # if user didn't pick a file don't continue
            self.isoFileName = str(self.isoFile[0])
            self.isoValue.setText(self.isoFileName)
        else:
            self.Status.addItem("No Molfit file selected!")
            self.Status.scrollToBottom()
            print "No Molfit file selected!"
        
 
    def set_center(self):
        self.CenterList.clear()
        self.CenterList.addItem(str(self.CenterSlider.value()))

    def set_width(self):
        self.WidthList.clear()
        self.WidthList.addItem(str(self.WidthSlider.value()))

    def get_cont(self):
        self.contLevel = float(self.ContLevel.text())

    def get_vlsr(self):
        self.vLSR = float(self.inputvLSR.text())

    def get_size(self):
        self.sizeValue = float(self.SizeValue.text())
#        print self.contLevel

    def get_fmin(self):
        self.dfmin = float(self.FreqMin.text())
#        print self.dfmin

    def get_fmax(self):
        self.dfmax = float(self.FreqMax.text())
#        print self.dfmax

    def get_fstep(self):
        self.dfstep = float(self.FreqStep.text())
#        print self.dfstep



    def rmmpl(self,):
        self.mplvl.removeWidget(self.canvas)
        self.canvas.close()
        self.mplvl.removeWidget(self.toolbar)
        self.toolbar.close()

    def exec_fit_1(self):
        self.exec_fit(1)

    def exec_fit_2(self):
        self.exec_fit(2)

    def exec_fit_3(self):
        self.exec_fit(3)

    def exec_fit_4(self):
        self.exec_fit(4)


    def exec_fit(self, n):
#        ax1f1 = fig1.add_subplot(111)
        #    ax1f1.plot(self.dfreq, self.data)
        iso_flag = self.isoButton.isChecked()
        if (iso_flag):
            self.Status.addItem("Isotopologues considered from file " + self.isoFileName)
        else:
            self.Status.addItem("No Isotopologues considered")

        TelescopeSize=self.sizeValue
        InterFlag = self.interButton.isChecked()
        if (InterFlag):
            self.Status.addItem("Interferometer mode assumed, resolution " + str(self.sizeValue))
        else:
            self.Status.addItem("Single dish mode assumed, size " +str(self.sizeValue))

        self.Status.scrollToBottom()

        t_back_flag = True        
        tBack = self.contLevel
        tslope = 0.0000000000E+00
        nH_flag = True
        N_H = 3.0000000000E+22
        beta_dust = 0.0
        kappa_1300 = 0.002
        IsoTableFileName = self.isoFileName
        NumberProcessors = 1
        vLSR=self.vLSR
        RestFreq = (self.dfmax-self.dfmin)/2.
 
#        print  self.MolfitList.count(), n
#        print self.MolfitList.item(0).text()
#        print self.MolfitList.item(1).text()
#        if (n == 1):
#            print "n=1"
#        if (self.MolfitList.count() > 0):
#            print "Listcount > 0"

        if (n == 1 and self.MolfitList.count() > 0):
#            print "1"
            MolFileName =  str(self.MolfitList.item(0).text())
        elif (n == 2 and  self.MolfitList.count() > 1):
            MolFileName =  str(self.MolfitList.item(1).text())
        elif (n == 3 and  self.MolfitList.count() > 2):
            MolFileName =  str(self.MolfitList.item(2).text())
        elif (n == 4 and  self.MolfitList.count() > 3):
            MolFileName =  str(self.MolfitList.item(3).text())
        else:
            self.Status.addItem("No Molfit file defined!")
            print "no Molfit Files defined!"
 
        try:
            MolFileName.isalpha()
            modeldata, log, trans, inttau, jobdir = tX.myXCLASS(self.dfmin, self.dfmax, self.dfstep, TelescopeSize, InterFlag, t_back_flag, tBack, tslope, nH_flag, N_H, beta_dust, kappa_1300, MolFileName, iso_flag, IsoTableFileName, RestFreq, vLSR)
            if (n == 1 and  self.MolfitList.count() > 0):
#                print "here we are"
                self.ffreq_1 = modeldata[:,0]
                self.fit_1 = modeldata[:,2]
#                print "found:",  self.ffreq_1, self.fit_1
            elif (n == 2 and  self.MolfitList.count() > 1):
                self.ffreq_2 = modeldata[:,0]
                self.fit_2 = modeldata[:,2]
            elif (n == 3 and  self.MolfitList.count() > 2):
                self.ffreq_3 = modeldata[:,0]
                self.fit_3 = modeldata[:,2]
            elif (n == 4 and  self.MolfitList.count() > 3):
                self.ffreq_4 = modeldata[:,0]
                self.fit_4 = modeldata[:,2]
            else:
                pass
        except:
            self.Status.addItem("No Molfit file found!")
            print "no MolFit File found!"
        self.re_plot()

    def first_plot(self):
        try:
            self.rmmpl()
        except:
            pass
        self.fig1 = Figure()
        self.ax1f1 = self.fig1.add_subplot(111)
        self.ax1f1.plot(self.dfreq, self.data, drawstyle = 'steps-mid', label="Data",lw=2)
        self.ax1f1.legend()
        self.ax1f1.set_xlabel("Frequency [MHz]")
        self.ax1f1.set_ylabel("Intensity [K]")
        self.ax1f1.set_xlim(self.dfmin, self.dfmax)
        self.ax1f1.margins(x=0.05, y=0.05)
        self.ax1f1.grid()
        self.addmpl(self.fig1)
        self.show()
        
    def re_plot(self):
        try:
            self.rmmpl()
        except:
            pass
        self.fig1 = Figure()
        self.ax1f1 = self.fig1.add_subplot(111)
        try:
            self.dfreq.real
            self.ax1f1.plot(self.dfreq, self.data, drawstyle = 'steps-mid', label="Data",lw=2)
        except:
            pass
        
#        print "replot"
#        print self.ffreq_1
#        print self.fit_1
        try:
            self.ffreq_1.real
            self.ax1f1.plot(self.ffreq_1, self.fit_1, drawstyle = 'steps-mid', label="Model 1",lw=2)
        except:
            pass
        try:
            self.ffreq_2.real
            self.ax1f1.plot(self.ffreq_2, self.fit_2, drawstyle = 'steps-mid', label="Model 2",lw=2)
        except:
            pass
        try:
            self.ffreq_3.real
            self.ax1f1.plot(self.ffreq_3, self.fit_3, drawstyle = 'steps-mid', label="Model 3",lw=2)
        except:
            pass
        try:
            self.ffreq_4.real
            self.ax1f1.plot(self.ffreq_4, self.fit_4, drawstyle = 'steps-mid', label="Model 4",lw=2)
        except:
            pass
        self.ax1f1.legend()
        self.ax1f1.set_xlabel("Frequency [MHz]")
        self.ax1f1.set_ylabel("Intensity [K]")
        self.ax1f1.grid()
        self.ax1f1.margins(x=0.05, y=0.05)
#        Cpos = self.CenterSlider.sliderPosition()
#        Cwid = self.WidthSlider.sliderPosition()
#        fmin = Cpos-Cwid/2
#        fmax = Cpos+Cwid/2
#        self.ax1f1.set_xlim(fmin, fmax)
        self.addmpl(self.fig1)
        self.show()

#        main = Main()
#        main.addmpl(fig1)
#        main.show()

if __name__ == '__main__':
    import sys
    from PyQt4 import QtGui
    
    app = QtGui.QApplication(sys.argv)
    main = Main()
    main.show()
    sys.exit(app.exec_())  
